
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class V1::ObjectTypesController < ApplicationController
  include ::Paginator

  before_action :authenticate_user!, except: [:index, :show]
  before_action :set_object_type, only: [:show, :update, :destroy]
  before_action :authorize!, only: [:update, :destroy]


  # GET /object_types
  # GET /object_types.json
  def index
    object_types = ObjectType.includes(:mime_types)
    render json: object_types
  end

  # GET /learning_objects/1
  # GET /learning_objects/1.json
  def show
    render json: @object_type
  end

  # POST /object_types
  # POST /object_types.json
  def create
    @object_type = ObjectType.new(sanitazed_params)
    authorize @object_type

    if @object_type.save
      render json: @object_type, status: :created
    else
      render json: @object_type.errors, status: :unprocessable_entity
    end
  end

  # PATCH/PUT /object_types/1
  # PATCH/PUT /object_types/1.json
  def update
    if @object_type.update(sanitazed_params)
      render json: @object_type, status: :ok
    else
      render json: @object_type.errors, status: :unprocessable_entity
    end
  end

  # DELETE /object_types/1
  # DELETE /object_types/1.json
  def destroy
    SafeObjectTypeDestroyService.execute @object_type
    render status: :ok
  end

  private

  # Use callbacks to share common setup or constraints between actions.
  def set_object_type
    @object_type = ObjectType.find(params[:id])
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def object_type_params
    params.require(:object_type).permit(:name, mime_types: [])
  end

  # Build mimetypes
  def sanitazed_params
    mime_params = object_type_params
    mime_params[:mime_types] = []
    unless object_type_params[:mime_types].blank?
      object_type_params[:mime_types].each do |mime_type|
        mime_params[:mime_types] << MimeType.find(mime_type.to_i)
      end
    end
    mime_params
  end

  def authorize!
    authorize @object_type
  end
end
