
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

module PackageService
  class Link
    def initialize(objects)
      @generator = Generator.new(objects)
    end

    def link
      link = get_package_link
      if link.nil?
        @generator.generate
        link = retrieve_link
      end
      link
    end

    private

    def get_package_link
      package = PackageItem.select("package_id").where(:packageable => @generator.objects).group("package_id")
      return nil if package.blank?
      link = Package.find(package.first.package_id).file_path
      if !File.exist?(Rails.root.join('public',link))
        Package.destroy(package.first.package_id)
        return nil
      end
      link
    end

    def retrieve_link
      if wait_job
        filename = @generator.filename

        filepath = PackageService.dirname+'/'+filename
        package = Package.create(file_path: filepath)
        package.add_items(@generator.objects)
        return package.file_path
      end

      nil
    end

    def wait_job
      unless File.exist?(PackageService.file_root(@generator.filename))
        Timeout.timeout(60) do
          begin
            sleep(1.0)
          end until File.exist?(PackageService.file_root(@generator.filename))
        end
      end
      true
    rescue Timeout::Error
      false
    end
  end
end
