
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

namespace :score do
  desc 'calculate all objects async'
  task calculate_all: :environment do
    %w(LearningObject Collection User).map { |o| calculate_async(o) }
  end

  desc 'calculate score sync'
  task :calculate_sync, [:klass] => :environment do |_t, args|
    return false unless %w(User LearningObject Collection).include? args.klass
    klass = args.klass.constantize
    progressbar = ProgressBar.create(title: 'calculate LearningObjects score', total: klass.count)
    Parallel.each(klass.all, in_threads: 8) do |lo|
      ActiveRecord::Base.connection_pool.with_connection do
        calculate_sync(lo)
      end
      progressbar.increment
    end
  end

  desc 'calculate all from klass'
  task :calculate_sync, [:klass] => :environment do |_t, args|
    return false unless %w(User LearningObject Collection).include? args.klass

    klass = args.klass.constantize

    progressbar = ProgressBar.create(title: "calculating score of #{klass.name} objects", total: klass.count)
    # klass.find_each do |object|
    Parallel.each(klass.all, in_threads: 8) do |object|
      ActiveRecord::Base.connection_pool.with_connection do
        calculate_sync(object)
      end
      progressbar.increment
    end
  end

  private

  def calculate_async(object)
    return false unless %w(Collection LearningObject User).include?(object)

    object.constantize.all.find_each do |item|
      ScoreCalculatorWorker.perform_async(item.id, item.class.name)
    end
  end

  def calculate_sync(object)
    calculator = ScoreCalculatorService.new(object)

    score = calculator.calculate

    object.update(score: score) if score.is_a? Float

    object.update(user_category_id: calculator.category_by_reputation) if object.is_a? User
  end
end
