# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'json'

namespace :tag do
  desc 'Generate tag clusters'
  task :generate_clusters => [:environment] do
    include TagSearchService

    graph_path = TagSearchService.file_path+".net"
    # Create hash of tag co occurrence
    hash = create_hash()
    # Create a graph infomap can read
    create_pajek_net_graph(hash, graph_path)

    infomap_ftree(graph_path, TagSearchService.root_dir)

    # Cluster needs to be read from disk again, so clear cache of TagSearchService
    Rails.cache.delete(TagSearchService::FTREE_CACHE_KEY)
    TagSearchService::parse_ftree_cache()
  end # task

  private

  # hash[id1][id2] will equal how many times tags with id1 and id2 appear together on a LO
  def create_hash()
    hash = {}

    LearningObject.all.each do |lo|
      # for each lo, count tags and tag pairs and add to hash
      # if id1 <= id2
      lo.tags.each.with_index do |t, i|
        # initialize values
        hash[t.id] = {} if hash[t.id].nil?
        hash[t.id][t.id] = 0 if hash[t.id][t.id].nil?

        hash[t.id][t.id] += 1 # add self occurrence

        # for each next tags (with higher index than t)
        lo.tags.drop(i+1).each do |t2|
          # swap if necessary ([t1][t2], t1 should always have lower id)
          if t.id > t2.id
            # swaps t with t2
            t, t2 = t2, t
            # check nil
            hash[t.id] = {} if hash[t.id].nil?
          end
          # initialize value
          hash[t.id][t2.id] = 0 if hash[t.id][t2.id].nil?

          hash[t.id][t2.id] += 1 # add co occurrence
        end
      end
    end

    return hash
  end

  def create_pajek_net_graph(hash, path)
    File.open(path, "w+") do |f|
      f << "*Vertices #{Tag.all.size}\n"

      tag_index = {}
      Tag.all.each_with_index do |t,i|
        f << "#{i+1} \"#{t.name}\"\n"
        tag_index[t.id] = i+1
      end

      f << "*Edges\n"

      hash.each do |id1, ids2Hash|
        ids2Hash.each do |id2, value|
          if id1 != id2
            f << "#{tag_index[id1]} #{tag_index[id2]} \
            #{hash[id1][id2].to_f/(Math.sqrt(hash[id1][id1]*hash[id2][id2]))}\n"
            # weight of the edge is the cos distance
          end
        end
      end
    end
  end

  def infomap_ftree(graph_path, out_dir)
    system("infomap --ftree #{graph_path} #{TagSearchService.root_dir}")
    if $?.exitstatus != 0
      puts "-- Error on Infomap call"
      puts "-- Make sure you have infomap in your executable path"
      exit
    end
  end
end # namespace
