
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'test_helper'

class V1::ReviewsControllerTest < ActionController::TestCase
  tests V1::ReviewsController
  include Devise::Test::ControllerHelpers

  test 'should user review a learning object and return :created code' do
    auth_request users(:admin)
    assert learning_objects(:user_lo).id > 0
    create_review learning_objects(:user_lo)
    assert_response :created
  end

  test 'should user review a collection and return :created code' do
    auth_request users(:admin)
    create_review collections(:ufpr)
    assert_response :created
  end

  test 'should guest user review a collection and return :unauthorized code' do
    create_review collections(:ufpr)
    assert_response :unauthorized
  end

  test 'should user delete review withou permission and return :unauthorized code' do
    auth_request users(:john)
    delete :destroy, params: { collection_id: collections(:ufpr).id, id: reviews(:collection).id }
    assert_response :unauthorized
  end

  private

  def create_review(object)
    object_param_name = "#{object.class.to_s.downcase}_id".to_sym
    object_param_name = 'learning_object_id' if LearningObject == object.class
    post :create, params: { review: {
      name: 'Test of review',
      description: 'testing',
      pros: 'my pros',
      cons: 'my cons',
      review_ratings_attributes: [
        { rating_id: ratings(:usability).id, value: 3 },
        { rating_id: ratings(:application_context).id, value: 3 },
        { rating_id: ratings(:content).id, value: 3 }
      ]
    }, object_param_name => object.id }
  end
end
