
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'test_helper'

class V1::SearchControllerTest < ActionController::TestCase
  tests V1::SearchController

  setup do
    SearchService::SEARCH_CLASSES.each { |search_class| reindex search_class.constantize }
    auth_application
  end

  test 'should search return :ok' do
    SearchService::SEARCH_CLASSES.each do |search_class|
      get :index, params: { search_class: search_class }
      assert_response :ok
    end
  end

  test 'should search with invalid class and return :bad_request' do
    get :index, params: { search_class: 'invalid' }
    assert_response :bad_request
  end

  test 'should autocomplete with invalid class and return :bad_request' do
    get :autocomplete, params: { search_class: 'invalid' }
    assert_response :bad_request
  end

  private

  def reindex(klass)
    klass.reindex
    klass.searchkick_index.refresh
  end

end
