
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'test_helper'

class V1::UsersControllerTest < ActionController::TestCase
  tests V1::UsersController
  include Devise::Test::ControllerHelpers

  test 'should get index' do
    skip('Unsolved issue: ArgumentError: wrong number of arguments (2 for 0)')
  end

  test 'should post user to create and return :created code' do
    auth_request users(:jack)
    post :create, params: { user: { name: 'mauricio',
                         email: 'testando@inf.ufpr.br',
                         password: 'test123455',
                         description: 'usuario super legal',
                         password_confirmation: 'test123455',
                         terms_of_service: '1' } }
    assert_response :created
  end

  test 'should post invalid user to create and return :unprocessable_entity code' do
    auth_request users(:jack)
    post :create, params: { user: { name: 'mauricio', email: '', password: 'test123455', password_confirmation: 'test123455',
                         terms_of_service: false, role_ids: [] } }
    assert_response :unprocessable_entity
  end

  test 'should put user to update and return :ok code' do
    auth_request users(:jack)
    user = users(:john)
    put :update, params: { user: { name: 'mauricio',
                        email: 'testando@inf.ufpr.br',
                        password: 'test123455',
                        description: 'sou legal',
                        password_confirmation: 'test123455',
                        terms_of_service: '1'
    }, id: user.id }
    assert_response :ok
  end

  test 'should put invalid user to update and return :unprocessable_entity code' do
    auth_request users(:jack)
    user = users(:john)
    put :update, params: { user: { name: 'mauricio', email: '', password: 'test123455', password_confirmation: 'test123455',
                        terms_of_service: false }, id: user.id }
    assert_response :unprocessable_entity
  end

  test 'should delete an user and return :ok code' do
    auth_request users(:jack)
    user = users(:john)
    delete :destroy, params: { id: user.id }
    assert_response :ok
  end

  test 'should not current user delete yourself' do
    auth_request users(:john)
    user = users(:john)
    delete :destroy, params: { id: user.id }
    assert_response :forbidden
  end

  test 'should user follow an user jack' do
    auth_request users(:john)
    post :follow, params: { id: users(:jack).id }
    assert_response :created
  end

  test 'should user follow an already followed user and return :forbidden code' do
    auth_request users(:john)

    post :follow, params: { id: users(:jack).id }
    assert_response :created

    post :follow, params: { id: users(:jack).id }
    assert_response :forbidden
  end

  test 'should user unfollow an user jack' do
    auth_request users(:john)
    post :follow, params: { id: users(:jack).id }
    assert_response :created

    delete :unfollow, params: { id: users(:jack).id }
    assert_response :ok
  end

end
