
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

# == Schema Information
#
# Table name: learning_objects
#
#  id                     :integer          not null, primary key
#  id_dspace              :integer
#  name                   :string
#  author                 :string
#  description            :text
#  published_at           :datetime
#  score                  :float            default("0.0")
#  school_level           :integer
#  metadata               :jsonb            default("{}")
#  keywords               :text
#  publisher_id           :integer
#  publisher_type         :string
#  language_id            :integer
#  object_type_id         :integer
#  created_at             :datetime         not null
#  updated_at             :datetime         not null
#  views_count            :integer          default("0")
#  downloads_count        :integer          default("0")
#  likes_count            :integer          default("0")
#  shares_count           :integer          default("0")
#  state                  :string           default("published")
#  thumbnail_file_name    :string
#  thumbnail_content_type :string
#  thumbnail_file_size    :integer
#  thumbnail_updated_at   :datetime
#  attachment_id          :integer
#

require 'test_helper'

class LearningObjectTest < ActiveSupport::TestCase

  should have_many :collection_items
  should have_many(:collections).through(:collection_items)
  should have_many :complaints
  should have_many(:attachments).class_name('LearningObject::Attachment')

  should belong_to :publisher
  should belong_to :language
  should belong_to(:attachment).class_name('LearningObject::Attachment')
  should belong_to :object_type

  #sociable
  should have_many :views
  should have_many :downloads
  should have_many :likes
  should have_many :shares

  #taggable
  should have_many :taggings
  should have_many(:tags).through(:taggings)

  include ::Portalmec::SociableTests

  test 'should search data return a hash' do
    learning_object = learning_objects(:lo_complete)
    assert_equal learning_object.search_data, {
        name: 'Institution Object 1',
        description: 'Testing',
        author: 'Mauricio',
        object_type: 'Imagem',
        score: learning_object.score,
        published_at: learning_object.published_at,
        tags: learning_object.tags.map(&:name),
        subjects: learning_object.subjects.pluck(:id),
        educational_stages: learning_object.educational_stages.pluck(:id),
        source: nil,
        state: LearningObject.states[learning_object.state]
    }
  end

  test 'should get all learning objects created this week' do
    LearningObject.this_week.each { |o| assert_instance_of LearningObject, o }
  end

  test 'should get all learning objects created this month' do
    LearningObject.this_month.each { |o| assert_instance_of LearningObject, o }
  end

  protected

  def sociable_object
    learning_objects(:user_lo)
  end

  def sociable_user
    users(:john)
  end
end
